@@ -1,8 +1,8 @@
-__version__ = '4.2.0'
+__version__ = '5.1.1'
 
 from functools import wraps
 
-from tri_declarative import LAST
+import django
 
 from iommi._db_compat import (
     register_factory,
@@ -11,6 +11,10 @@ from iommi._db_compat import (
 from iommi.action import Action
 from iommi.asset import Asset
 from iommi.base import MISSING
+from iommi.edit_table import (
+    EditColumn,
+    EditTable,
+)
 from iommi.form import (
     Field,
     Form,
@@ -37,6 +41,7 @@ from iommi.query import (
     Query,
     register_filter_factory,
 )
+from iommi.sort_after import LAST
 from iommi.style import (
     register_style,
     Style,
@@ -53,9 +58,18 @@ setup_db_compat()
 
 def render_if_needed(request, response):
     if isinstance(response, Part):
-        if not response._is_bound:
-            response = response.bind(request=request)
-        return response.render_to_response()
+        try:
+            if not response._is_bound:
+                response = response.bind(request=request)
+            return response.render_to_response()
+        except Exception as e:
+            filename, lineno = response._instantiated_at_info
+            from iommi.synthetic_traceback import SyntheticException
+            fake = SyntheticException(tb=[
+                dict(filename=filename, f_lineno=lineno, function='<iommi declaration>', f_globals={}, f_locals={})
+            ])
+
+            raise e from fake
     else:
         return response
 
@@ -89,6 +103,8 @@ __all__ = [
     'Action',
     'Asset',
     'Column',
+    'EditTable',
+    'EditColumn',
     'Field',
     'Fragment',
     'Form',
@@ -116,4 +132,5 @@ __all__ = [
     'iommi_render',
 ]
 
-default_app_config = 'iommi.apps.IommiConfig'
+if django.VERSION[:2] < (3, 2):
+    default_app_config = 'iommi.apps.IommiConfig'
