@@ -7,14 +7,6 @@ from typing import (
 )
 
 from django.utils.translation import gettext_lazy
-from tri_declarative import (
-    class_shortcut,
-    dispatch,
-    EMPTY,
-    Refinable,
-    setdefaults_path,
-    with_meta,
-)
 
 from iommi._web_compat import (
     format_html,
@@ -25,23 +17,30 @@ from iommi.base import (
     capitalize,
     values,
 )
+from iommi.declarative.namespace import setdefaults_path
+from iommi.declarative.with_meta import with_meta
 from iommi.fragment import (
     Fragment,
     Tag,
 )
 from iommi.member import Members
 from iommi.part import Part
-from iommi.refinable import EvaluatedRefinable
+from iommi.refinable import (
+    EvaluatedRefinable,
+    Refinable,
+)
+from iommi.shortcut import with_defaults
 
 
 @with_meta
 class Action(Fragment):
+    # language=rst
     """
     The `Action` class describes buttons and links.
 
     Examples:
 
-    .. code:: python
+    .. code-block:: python
 
         # Link
         Action(attrs__href='http://example.com')
@@ -86,17 +85,18 @@ class Action(Fragment):
             class Meta:
                 actions__submit = Action.primary(post_handler=on_submit)
 
-    .. test
+        # @test
         r = req('post', **{'-submit': ''})
         MyForm().bind(request=r).render_to_response()
         MyOtherForm().bind(request=r).render_to_response()
+        # @end
     """
 
     group: str = EvaluatedRefinable()
     display_name: str = EvaluatedRefinable()
     post_handler: Callable = Refinable()
 
-    @dispatch(
+    @with_defaults(
         tag='a',
         display_name=lambda action, **_: capitalize(action._name).replace('_', ' '),
     )
@@ -129,41 +129,36 @@ class Action(Fragment):
         return self.own_target_marker() in self.iommi_parent().iommi_parent()._request_data
 
     @classmethod
-    @class_shortcut(
+    @with_defaults(
         tag='button',
     )
-    def button(cls, call_target=None, **kwargs):
-        return call_target(**kwargs)
+    def button(cls, **kwargs):
+        return cls(**kwargs)
 
     @classmethod
-    @class_shortcut(
-        call_target__attribute='button',
+    @with_defaults(
         attrs__accesskey='s',
         attrs__name=lambda action, **_: action.own_target_marker(),
         display_name=gettext_lazy('Submit'),
     )
-    def submit(cls, call_target=None, **kwargs):
-        return call_target(**kwargs)
+    def submit(cls, **kwargs):
+        return cls.button(**kwargs)
 
     @classmethod
-    @class_shortcut(
-        call_target__attribute='submit',
-    )
-    def primary(cls, call_target=None, **kwargs):
-        return call_target(**kwargs)
+    @with_defaults
+    def primary(cls, **kwargs):
+        return cls.submit(**kwargs)
 
     @classmethod
-    @class_shortcut(
-        call_target__attribute='submit',
-    )
-    def delete(cls, call_target=None, **kwargs):
-        return call_target(**kwargs)
+    @with_defaults
+    def delete(cls, **kwargs):
+        return cls.submit(**kwargs)
 
     @classmethod
-    @class_shortcut(
-        icon_classes=[],
-    )
-    def icon(cls, icon, *, display_name=None, call_target=None, icon_classes=None, **kwargs):
+    @with_defaults
+    def icon(cls, icon, *, display_name=None, icon_classes=None, **kwargs):
+        if icon_classes is None:
+            icon_classes = []
         icon_classes_str = ' '.join(['fa-' + icon_class for icon_class in icon_classes]) if icon_classes else ''
         if icon_classes_str:
             icon_classes_str = ' ' + icon_classes_str
@@ -171,7 +166,7 @@ class Action(Fragment):
             kwargs,
             display_name=format_html('<i class="fa fa-{}{}"></i> {}', icon, icon_classes_str, display_name),
         )
-        return call_target(**kwargs)
+        return cls(**kwargs)
 
 
 def group_actions(actions: Dict[str, Action]):
@@ -192,5 +187,6 @@ def group_actions(actions: Dict[str, Action]):
 
 
 class Actions(Members, Tag):
-    attrs: Attrs = Refinable()  # attrs is evaluated, but in a special way so gets no EvaluatedRefinable type
+    # attrs is evaluated, but in a special way so gets no EvaluatedRefinable type
+    attrs: Attrs = Refinable()
     tag = EvaluatedRefinable()
