@@ -1,9 +1,9 @@
 import functools
-from functools import wraps
 from typing import Type
 from urllib.parse import urlencode
 
 from django.apps import apps as django_apps
+from django.conf import settings
 from django.contrib import (
     auth,
     messages,
@@ -11,43 +11,51 @@ from django.contrib import (
 from django.contrib.auth.hashers import check_password
 from django.contrib.auth.password_validation import validate_password
 from django.core.exceptions import ValidationError
+from django.db.models import Model
 from django.http import (
     Http404,
     HttpResponseRedirect,
 )
+from django.shortcuts import (
+    redirect,
+    resolve_url,
+)
 from django.urls import (
     path,
     reverse,
 )
 from django.utils.safestring import mark_safe
 from django.utils.translation import gettext
-from tri_declarative import (
-    class_shortcut,
-    dispatch,
-    EMPTY,
-    LAST,
-    Namespace,
-    setdefaults_path,
-    with_meta,
-)
-from tri_struct import Struct
 
 from iommi import (
+    Action,
     Field,
     Form,
     Fragment,
-    Header,
     html,
+    LAST,
     Menu,
     MenuItem,
     Page,
     Table,
 )
+from iommi._web_compat import format_html
 from iommi.base import (
+    build_as_view_wrapper,
     items,
     values,
 )
+from iommi.declarative.dispatch import dispatch
+from iommi.declarative.namespace import (
+    EMPTY,
+    flatten,
+    Namespace,
+    setdefaults_path,
+)
+from iommi.declarative.with_meta import with_meta
 from iommi.refinable import Refinable
+from iommi.shortcut import with_defaults
+from iommi.struct import Struct
 
 app_verbose_name_by_label = {
     config.label: config.verbose_name.replace('_', ' ')
@@ -61,17 +69,6 @@ joined_app_name_and_model = {
 }
 
 
-def require_login(view):
-    @wraps(view)
-    def wrapper(cls, request, *args, **kwargs):
-        if not getattr(request, 'user', None) or not request.user.is_authenticated:
-            return HttpResponseRedirect(f'{reverse(Auth.login)}?{urlencode(dict(next=request.path))}')
-
-        return view(cls, request, *args, **kwargs)
-
-    return wrapper
-
-
 @with_meta
 class Messages(Fragment):
     class Meta:
@@ -129,31 +126,52 @@ class Admin(Page):
         iommi_style = 'bootstrap'
         table_class = Table
         form_class = Form
+        apps = EMPTY
+        parts = EMPTY
         apps__auth_user__include = True
         apps__auth_group__include = True
         parts__messages = Messages()
         parts__list_auth_user = dict(
             auto__include=['username', 'email', 'first_name', 'last_name', 'is_staff', 'is_active', 'is_superuser'],
             columns=dict(
-                username__filter__freetext=True,
-                email__filter__freetext=True,
-                first_name__filter__freetext=True,
-                last_name__filter__freetext=True,
+                username__filter=dict(
+                    include=True,
+                    freetext=True,
+                ),
+                email__filter=dict(
+                    include=True,
+                    freetext=True,
+                ),
+                first_name__filter=dict(
+                    include=True,
+                    freetext=True,
+                ),
+                last_name__filter=dict(
+                    include=True,
+                    freetext=True,
+                ),
+
                 is_staff__filter__include=True,
                 is_active__filter__include=True,
                 is_superuser__filter__include=True,
             ),
         )
 
+    model: Type[Model] = Refinable()
+    instance: Model = Refinable()
+    app_name: str = Refinable()
+    model_name: str = Refinable()
+    operation: str = Refinable()
     table_class: Type[Table] = Refinable()
     form_class: Type[Form] = Refinable()
 
-    apps: Namespace = Refinable()  # Global configuration on apps level
+    # Global configuration on apps level
+    apps: Namespace = Refinable()
 
     menu = Menu(
         sub_menu=dict(
             root=MenuItem(
-                url=lambda admin, **_: reverse(admin.__class__.all_models), display_name=gettext('iommi administration')
+                url=lambda admin, **_: reverse('iommi.Admin.all_models'), display_name=gettext('iommi administration')
             ),
             change_password=MenuItem(
                 url=lambda **_: reverse(Auth.change_password), display_name=gettext('Change password')
@@ -163,10 +181,6 @@ class Admin(Page):
     )
 
     @read_config
-    @dispatch(
-        apps=EMPTY,
-        parts=EMPTY,
-    )
     def __init__(self, parts, apps, **kwargs):
         # Validate apps params
         for k in apps.keys():
@@ -174,8 +188,71 @@ class Admin(Page):
                 k in joined_app_name_and_model
             ), f'{k} is not a valid app/model key.\n\nValid keys:\n    ' + '\n    '.join(sorted(joined_app_name_and_model))
 
+        super(Admin, self).__init__(parts=parts, apps=apps, **kwargs)
+
+    def refine_with_params(self, app_name: str = None, model_name: str = None, pk: str = None):
+        refined_admin = self.refine(app_name=app_name, model_name=model_name)
+
+        model = django_apps.all_models[app_name][model_name] if app_name and model_name else None
+        try:
+            instance = model.objects.get(pk=pk) if pk is not None else None
+        except model.DoesNotExist:
+            raise Http404()
+
+        if model is not None and instance is None:
+            refined_admin = refined_admin.refine(model=model, parts__table__auto__model=model)
+
+        if instance is not None:
+            refined_admin = refined_admin.refine(instance=instance, parts__form__auto__instance=instance)
+
+        return refined_admin
+
+    def as_view(self):
+        def admin_view(request, *args, **kwargs):
+            if not getattr(request, 'user', None) or not request.user.is_authenticated:
+                return HttpResponseRedirect(f'{reverse(Auth.login)}?{urlencode(dict(next=request.path))}')
+
+            final_page = self.refine_with_params(
+                app_name=kwargs.pop('app_name', None),
+                model_name=kwargs.pop('model_name', None),
+                pk=kwargs.pop('pk', None),
+            ).refine_done()
+
+            if not self.has_permission(request, instance=final_page.instance, model=final_page.model, operation=final_page.operation):
+                raise Http404()
+
+            view = build_as_view_wrapper(final_page)
+
+            return view(request, *args, **kwargs)
+
+        return admin_view
+
+    def on_refine_done(self):
+        part_name = ''
+        part_name += self.operation
+        if self.app_name:
+            part_name += '_' + self.app_name
+        if self.model_name:
+            part_name += '_' + self.model_name
+
+        table = self.parts.get('table')
+        if table is not None:
+            setdefaults_path(
+                self.parts,
+                **{part_name: flatten(table)}
+            )
+            self.parts.table = None
+
+        form = self.parts.get('form')
+        if form is not None:
+            setdefaults_path(
+                self.parts,
+                **{part_name: flatten(form)}
+            )
+            self.parts.form = None
+
         def should_throw_away(k, v):
-            if isinstance(v, Namespace) and 'call_target' in v:
+            if k == part_name:
                 return False
 
             if k == 'all_models':
@@ -193,93 +270,135 @@ class Admin(Page):
 
             return False
 
-        parts = {
+        self.parts = {
             # Arguments that are not for us needs to be thrown on the ground
             k: None if should_throw_away(k, v) else v
-            for k, v in items(parts)
+            for k, v in items(self.parts)
         }
 
-        super(Admin, self).__init__(parts=parts, apps=apps, **kwargs)
+        super(Admin, self).on_refine_done()
 
     @staticmethod
     def has_permission(request, operation, model=None, instance=None):
-        return request.user.is_staff
+        if request.user.is_superuser:
+            return True
+
+        if not request.user.is_staff:
+            return False
+
+        if model is None:
+            return True
+
+        action = {
+            'create': 'add',
+            'edit': 'change',
+            'list': 'view',
+        }.get(operation, operation)
+
+        return request.user.has_perm(f'{model._meta.app_label}.{action}_{model._meta.model_name}')
 
     def own_evaluate_parameters(self):
         return dict(admin=self, **super(Admin, self).own_evaluate_parameters())
 
     @classmethod
-    @class_shortcut(
-        table=EMPTY,
-        table__call_target__attribute='div',
+    @with_defaults(
+        operation='all_models',
     )
-    @require_login
-    def all_models(cls, request, table, call_target=None, **kwargs):
-        if not cls.has_permission(request, operation='all_models'):
-            raise Http404()
-
-        def rows(admin, **_):
+    def all_models(cls, table=None, **kwargs):
 
+        def rows(admin, request, included_filter=False, **_):
             for app_name, models in items(django_apps.all_models):
                 has_yielded_header = False
 
                 for model_name, model in sorted(items(models), key=lambda x: x[1]._meta.verbose_name_plural):
-                    if not admin.apps.get(f'{app_name}_{model_name}', {}).get('include', False):
+                    key = f'{app_name}_{model_name}'
+                    included = admin.apps.get(key, {}).get('include', False)
+                    if included == included_filter:
+                        continue
+
+                    if not cls.has_permission(request, instance=None, model=model, operation='view'):
                         continue
 
                     if not has_yielded_header:
                         yield Struct(
                             name=app_verbose_name_by_label[app_name],
                             verbose_app_name=app_verbose_name_by_label[app_name],
+                            app_name=None,
+                            model_name=None,
                             url=None,
-                            format=lambda row, table, **_: Header(row.name, _name='invalid_name')
-                            .bind(parent=table)
-                            .__html__(),
+                            format=lambda row, table, **_: html.h2(row.name, _name='invalid_name').bind(parent=table).__html__(),
+                            key=None,
                         )
                         has_yielded_header = True
 
                     yield Struct(
                         verbose_app_name=app_verbose_name_by_label[app_name],
                         app_name=app_name,
+                        model_name=app_name,
                         name=model._meta.verbose_name_plural.capitalize(),
                         url='%s/%s/' % (app_name, model_name),
                         format=lambda row, **_: row.name,
+                        key=key,
                     )
 
         table = setdefaults_path(
             Namespace(),
-            table,
+            table if table is not None else {},
             title=gettext('All models'),
             call_target__cls=cls.get_meta().table_class,
+            call_target__attribute='div',
             sortable=False,
             rows=rows,
-            header__template=None,
             page_size=None,
             columns__name=dict(
                 cell__url=lambda row, **_: row.url,
                 display_name='',
-                cell__format=lambda row, **kwargs: row.format(row=row, **kwargs),
+                cell__format=lambda row, **format_kwargs: row.format(row=row, **format_kwargs),
             ),
         )
 
-        return call_target(parts__all_models=table, **kwargs)
+        add_models = setdefaults_path(
+            Namespace(),
+            include=settings.DEBUG,
+            call_target__cls=cls.get_meta().table_class,
+            sortable=False,
+            rows=functools.partial(rows, included_filter=True),
+            page_size=None,
+            columns__conf=cls.get_meta().table_class.get_meta().member_class(
+                cell__value=lambda row, **_: f'apps__{row.key}__include = True' if row.key else '',
+                # cell__url=lambda row, **_: f'{row.url}add_model/' if row.key else None,
+                after=LAST,
+            ),
+            columns__name=dict(
+                display_name='',
+                cell__format=lambda row, **format_kwargs: row.format(row=row, **format_kwargs),
+            ),
+        )
+
+        return cls(
+            parts=dict(
+                all_models=table,
+
+                add_models_title=html.h1(gettext('Add models to admin'), include=settings.DEBUG),
+                add_models_help_text=html.p(format_html('''
+                Copy the conf value to the `Meta` class of an `iommi_admin.py` file.
+                
+                Read <a href="https://docs.iommi.rocks/en/latest/admin.html#customization">the docs for admin customization</a> for more information.
+                ''')),
+                add_models=add_models,
+            ),
+            **kwargs,
+        )
 
     @classmethod
-    @class_shortcut(
-        table=EMPTY,
+    @dispatch(
+        operation='list',
     )
-    @require_login
-    def list(cls, request, app_name, model_name, table, call_target=None, **kwargs):
-        model = django_apps.all_models[app_name][model_name]
-
-        if not cls.has_permission(request, operation='list', model=model):
-            raise Http404()
-
+    def list(cls, table=None, **kwargs):
         table = setdefaults_path(
             Namespace(),
-            table,
+            table if table is not None else {},
             call_target__cls=cls.get_meta().table_class,
-            auto__model=model,
             columns=dict(
                 select__include=True,
                 edit=dict(
@@ -291,99 +410,95 @@ class Admin(Page):
                     call_target__attribute='delete',
                     after=LAST,
                     cell__url=lambda row, **_: '%s/delete/' % row.pk,
+                    include=lambda request, table, **_: cls.has_permission(request, instance=None, model=table.model, operation='delete')
                 ),
             ),
             actions=dict(
                 create=dict(
-                    display_name=gettext('Create %(model_name)s') % dict(model_name=model._meta.verbose_name),
+                    display_name=lambda page, **_: gettext('Create %(model_name)s') % dict(model_name=page.model._meta.verbose_name),
                     attrs__href='create/',
                 ),
             ),
             query_from_indexes=True,
-            bulk__actions__delete__include=True,
+            bulk__actions__delete__include=lambda request, table, **_: cls.has_permission(request, instance=None, model=table.model, operation='delete'),
         )
 
-        return call_target(
+        return cls(
             parts__header__children__link__attrs__href='../..',
-            **{f'parts__list_{app_name}_{model_name}': table},
+            parts__table=table,
             **kwargs,
         )
 
     @classmethod
-    @class_shortcut(
-        form=EMPTY,
-    )
-    @require_login
-    def crud(cls, request, operation, form, app_name, model_name, pk=None, call_target=None, **kwargs):
-        model = django_apps.all_models[app_name][model_name]
-        instance = model.objects.get(pk=pk) if pk is not None else None
-
-        if not cls.has_permission(request, operation=operation, model=model, instance=instance):
-            raise Http404()
-
-        def on_save(form, instance, **_):
+    @with_defaults
+    def crud(cls, operation, form=None, **kwargs):
+        def on_save(request, form, instance, **_):
             message = f'{form.model._meta.verbose_name.capitalize()} {instance} was ' + (
                 'created' if form.extra.is_create else 'updated'
             )
             messages.add_message(request, messages.INFO, message, fail_silently=True)
 
-        def on_delete(form, instance, **_):
+        def on_delete(request, form, instance, **_):
             message = f'{form.model._meta.verbose_name.capitalize()} {instance} was deleted'
             messages.add_message(request, messages.INFO, message, fail_silently=True)
 
         form = setdefaults_path(
             Namespace(),
-            form,
+            form if form is not None else {},
             call_target__cls=cls.get_meta().form_class,
-            auto__instance=instance,
-            auto__model=model,
             call_target__attribute=operation,
             extra__on_save=on_save,
             extra__on_delete=on_delete,
+            actions__submit__include=lambda request, form, **_: cls.has_permission(request, instance=None, model=form.model, operation=operation),
+            editable=lambda request, form, **_: False if operation == 'delete' else cls.has_permission(request, instance=None, model=form.model, operation=operation)
         )
 
-        return call_target(
-            **{f'parts__{operation}_{app_name}_{model_name}': form},
+        return cls(
+            parts__form=form,
+            operation=operation,
             **kwargs,
         )
 
     @classmethod
-    @class_shortcut(
-        call_target__attribute='crud',
-        operation='create',
+    @with_defaults(
         parts__header__children__link__attrs__href='../../..',
     )
-    def create(cls, request, call_target, **kwargs):
-        return call_target(request=request, **kwargs)
+    def create(cls, **kwargs):
+        return cls.crud(
+            operation='create',
+            **kwargs,
+        )
 
     @classmethod
-    @class_shortcut(
-        call_target__attribute='crud',
-        operation='edit',
+    @with_defaults(
         parts__header__children__link__attrs__href='../../../..',
     )
-    def edit(cls, request, call_target, **kwargs):
-        return call_target(request=request, **kwargs)
+    def edit(cls, **kwargs):
+        return cls.crud(
+            operation='edit',
+            **kwargs,
+        )
 
     @classmethod
-    @class_shortcut(
-        call_target__attribute='crud',
-        operation='delete',
+    @with_defaults(
         parts__header__children__link__attrs__href='../../../..',
     )
-    def delete(cls, request, call_target, **kwargs):
-        return call_target(request=request, **kwargs)
+    def delete(cls, **kwargs):
+        return cls.crud(
+            operation='delete',
+            **kwargs,
+        )
 
     @classmethod
     def urls(cls):
         return Struct(
             urlpatterns=[
-                path('', cls.all_models),
-                path('<app_name>/<model_name>/', cls.list),
-                path('<app_name>/<model_name>/create/', cls.create),
-                path('<app_name>/<model_name>/<int:pk>/edit/', cls.edit),
-                path('<app_name>/<model_name>/<int:pk>/delete/', cls.delete),
-            ]
+                path('', cls.all_models().as_view(), name='iommi.Admin.all_models'),
+                path('<app_name>/<model_name>/', cls.list().as_view(), name='iommi.Admin.list'),
+                path('<app_name>/<model_name>/create/', cls.create().as_view(), name='iommi.Admin.create'),
+                path('<app_name>/<model_name>/<int:pk>/edit/', cls.edit().as_view(), name='iommi.Admin.edit'),
+                path('<app_name>/<model_name>/<int:pk>/delete/', cls.delete().as_view(), name='iommi.Admin.delete'),
+                        ]
             + Auth.urls().urlpatterns
         )
 
@@ -396,7 +511,7 @@ class Auth:
     @classmethod
     def logout(cls, request):
         auth.logout(request)
-        return HttpResponseRedirect('/')
+        return HttpResponseRedirect(resolve_url(settings.LOGOUT_REDIRECT_URL or '/'))
 
     @classmethod
     def change_password(cls, request):
