@@ -5,7 +5,7 @@ from os.path import (
     join,
 )
 
-from tri_struct import Struct
+from iommi.struct import Struct
 
 from iommi._web_compat import (
     format_html,
@@ -13,10 +13,7 @@ from iommi._web_compat import (
     mark_safe,
     Template,
 )
-from iommi.base import (
-    items,
-    values,
-)
+from iommi.base import items
 from iommi.member import (
     MemberBinder,
     Members,
@@ -142,6 +139,7 @@ def endpoint__debug_tree(endpoint, **_):
 
     request = HttpRequest()
     request.method = 'GET'
+    request.META = root._request.META
     if hasattr(root._request, 'user'):
         request.user = root._request.user
 
@@ -176,7 +174,7 @@ def should_ignore_frame(frame, env_paths):
 
     base_module_name = module_name.partition('.')[0]
     if base_module_name in (
-        'tri_declarative',
+        'iommi.declarative',
         'iommi',
         'django',
     ):
