@@ -3,16 +3,17 @@ from pathlib import Path
 from textwrap import dedent
 from typing import get_type_hints
 
-from tri_declarative import (
+from iommi import (
+    MISSING,
+    Part,
+)
+from iommi.base import items
+from iommi.declarative import get_declared
+from iommi.declarative.namespace import (
     flatten,
-    flatten_items,
-    get_declared,
-    get_shortcuts_by_name,
     Namespace,
 )
-
-from iommi import MISSING
-from iommi.base import items
+from iommi.shortcut import get_shortcuts_by_name
 
 
 def read_cookbook_links():
@@ -48,6 +49,8 @@ def get_default_classes():
     return [
         iommi.Table,
         iommi.Column,
+        iommi.EditTable,
+        iommi.EditColumn,
         iommi.Query,
         iommi.Filter,
         iommi.Form,
@@ -74,19 +77,28 @@ def get_default_classes():
     ]
 
 
-def generate_rst_docs(directory, classes=None):  # pragma: no cover - this is tested by rtd anyway
+def generate_api_docs_tests(directory, classes=None):  # pragma: no cover - this is tested by rtd anyway
     """
-    Generate documentation for tri.declarative APIs
+    Generate test files for tri.declarative APIs
 
-    :param directory: directory to write the .rst files into
-    :param classes: list of classes to generate documentation for
+    :param directory: directory to write the .py files into
+    :param classes: list of classes to generate tests for
     """
+    print(f'generate_api_docs_tests("{directory}")')
     if classes is None:
         classes = get_default_classes()
 
-    doc_by_filename = _generate_rst_docs(classes=classes)  # pragma: no mutate
+    doc_by_filename = _generate_tests_from_class_docs(classes=classes)  # pragma: no mutate
     for filename, doc in doc_by_filename:  # pragma: no mutate
-        with open(directory + filename, 'w') as f2:  # pragma: no mutate
+        # Avoid rewriting the files! If we do then pytest will redo the assertion rewriting which is very slow.
+        try:
+            with open(directory / filename) as f2:
+                old_contents = f2.read()
+            if old_contents == doc:
+                continue
+        except FileNotFoundError:
+            pass
+        with open(directory / filename, 'w') as f2:  # pragma: no mutate
             f2.write(doc)  # pragma: no mutate
 
 
@@ -98,7 +110,7 @@ def get_docs_callable_description(c):
     return c.__module__ + '.' + c.__name__
 
 
-def _generate_rst_docs(classes):
+def _generate_tests_from_class_docs(classes):
     import re
 
     cookbook_links = read_cookbook_links()
@@ -123,7 +135,7 @@ def _generate_rst_docs(classes):
         return (' ' * levels * 4) + s.strip()
 
     def get_namespace(c):
-        return Namespace({k: getattr(c.__init__, 'dispatch', {}).get(k) for k, v in items(get_declared(c, 'refinable'))})
+        return Namespace({k: getattr(c.__init__, '__iommi_with_defaults_kwargs', {}).get(k) for k, v in items(get_declared(c, 'refinable'))})
 
     for c in classes:
         from io import StringIO
@@ -134,13 +146,32 @@ def _generate_rst_docs(classes):
             f.write(indent(levels, s))
             f.write('\n')
 
-        def section(level, title):
+        def section(level, title, indent=0):
             underline = {0: '=', 1: '-', 2: '^', 3: '+'}[level] * len(title)
-            w(0, title)
-            w(0, underline)
-            w(0, '')
+            w(indent, title)
+            w(indent, underline)
+            w(indent, '')
 
-        section(0, c.__name__)
+        w(0, '''
+# NOTE: this file is automatically generated
+
+from iommi import *
+from iommi.admin import Admin
+from django.urls import (
+    include,
+    path,
+)
+from django.db import models
+from tests.helpers import req, user_req, staff_req
+from docs.models import *
+request = req('get')
+
+
+# language=rst
+"""
+        ''')
+
+        section(0, c.__name__, indent=0)
 
         class_doc = docstring_param_dict(c)
         constructor_doc = docstring_param_dict(c.__init__)
@@ -152,9 +183,45 @@ def _generate_rst_docs(classes):
 
         w(0, '')
 
+        w(0, '"""')
+        w(0, 'def test_base():')
+        w(1, '# language=rst')
+        w(1, '"""')
+
         if class_doc['text']:
-            f.write(class_doc['text'])
-            w(0, '')
+            in_code_block = False
+            code_block_indent = None
+            for line in class_doc['text'].split('\n'):
+                if line.strip().startswith('.. code-block:: python'):
+                    w(1, '"""')
+
+                    in_code_block = True
+                elif in_code_block:
+                    if code_block_indent is None:
+                        if line.strip():
+                            code_block_indent = len(line) - len(line.lstrip(' '))
+                        f.write(line)
+                        f.write('\n')
+                    else:
+                        if line.startswith(' ' * code_block_indent) or not line.strip():
+                            f.write(line)
+                            f.write('\n')
+                        else:
+                            in_code_block = False
+                            w(1, '# language=rst')
+                            w(1, '"""')
+                            w(1, line)
+                else:
+                    w(1, line)
+            if in_code_block:
+                w(0, '# language=rst')
+                w(0, '"""')
+            w(1, '')
+
+        w(1, '"""')
+        w(0, '')
+        w(0, '# language=rst')
+        w(0, '"""')
 
         if constructor_doc['text']:
             if class_doc['text']:
@@ -165,37 +232,6 @@ def _generate_rst_docs(classes):
 
         w(0, '')
 
-        refinable_members = sorted(dict.items(get_namespace(c)))
-        if refinable_members:
-            section(1, 'Refinable members')
-            type_hints = get_type_hints(c)
-            for refinable, value in refinable_members:
-                w(0, '* `' + refinable + '`')
-
-                if constructor_doc['params'].get(refinable):
-                    w(1, constructor_doc['params'][refinable])
-                    w(0, '')
-                type_hint = type_hints.get(refinable)
-                if type_hint:
-                    name = str(type_hint)
-                    if name.startswith('typing.'):
-                        name = name.replace('typing.', '')
-                    else:
-                        name = type_hint.__name__
-
-                    if type_hint in classes:
-                        w(1, f'Type: :doc:`{name}`')
-                    else:
-                        w(1, f'Type: `{name}`')
-                    w(1, '')
-
-                ref_name = f'{c.__name__}.{refinable}'
-                if ref_name in cookbook_name_by_refinable_name:
-                    w(1, f'Cookbook: :ref:`{ref_name.lower()}`')
-                    w(1, '')
-
-            w(0, '')
-
         defaults = Namespace()
         for refinable, value in sorted(get_namespace(c).items()):
             if value not in (None, MISSING):
@@ -206,23 +242,51 @@ def _generate_rst_docs(classes):
                 v = get_docs_callable_description(v)
 
                 if 'lambda' in v:
-                    v = v[v.find('lambda') :]
+                    v = v[v.find('lambda'):]
                     v = v.strip().strip(',').replace('\n', ' ').replace('  ', ' ')
+            if isinstance(v, Part):
+                v = v.bind()
             if v == '':
                 v = '""'
             return v
 
-        if defaults:
-            section(2, 'Defaults')
+        refinable_members = sorted(dict.items(get_namespace(c)))
+        if refinable_members:
+            section(1, 'Refinable members')
+            type_hints = get_type_hints(c)
+            for refinable, value in refinable_members:
+                w(0, '* `' + refinable + '`')
+                w(0, '')
 
-            for k, v in sorted(flatten_items(defaults)):
-                if v != {}:
-                    v = default_description(v)
+                if constructor_doc['params'].get(refinable):
+                    w(2, constructor_doc['params'][refinable])
+                    w(0, '')
+                type_hint = type_hints.get(refinable)
+                if type_hint:
+                    name = str(type_hint)
+                    if name.startswith('typing.'):
+                        name = name.replace('typing.', '')
+                    else:
+                        name = type_hint.__name__
+
+                    if type_hint in classes:
+                        w(2, f'Type: :doc:`{name}`')
+                    else:
+                        w(2, f'Type: `{name}`')
+                    w(1, '')
+
+                if refinable in defaults:
+                    w(2, f'Default: `{default_description(defaults.pop(refinable))}`')
+
+                ref_name = f'{c.__name__}.{refinable}'
+                if ref_name in cookbook_name_by_refinable_name:
+                    w(2, f'Cookbook: :ref:`{ref_name.lower()}`')
+                    w(2, '')
 
-                    w(0, '* `%s`' % k)
-                    w(1, '* `%s`' % v)
             w(0, '')
 
+        assert not defaults
+
         shortcuts = get_shortcuts_by_name(c)
         if shortcuts:
             section(1, 'Shortcuts')
@@ -236,7 +300,7 @@ def _generate_rst_docs(classes):
                     w(0, '')
                     w(0, '')
 
-                defaults = shortcut if isinstance(shortcut, dict) else shortcut.dispatch
+                defaults = shortcut if isinstance(shortcut, dict) else getattr(shortcut, '__iommi_with_defaults_kwargs', {})
                 if defaults:
                     defaults = Namespace(defaults)
                     section(3, 'Defaults')
@@ -246,4 +310,5 @@ def _generate_rst_docs(classes):
                         w(1, f'* `{v}`')
                     w(0, '')
 
-        yield '/%s.rst' % c.__name__, f.getvalue()
+        w(0, '''"""''')
+        yield 'test_doc__api_%s.py' % c.__name__, f.getvalue()
