@@ -1,8 +1,5 @@
-from tri_declarative import (
-    dispatch,
-    EMPTY,
-)
-
+from iommi.declarative.dispatch import dispatch
+from iommi.declarative.namespace import EMPTY
 
 class Errors:
     @dispatch(
