@@ -1,24 +1,24 @@
-import warnings
-from copy import (
-    copy,
-)
+from copy import copy
 from typing import (
     Dict,
     Type,
 )
 
-from tri_declarative import (
-    dispatch,
-    Namespace,
-    setdefaults_path,
-)
-from tri_struct import Struct
+from iommi.struct import Struct
 
 from iommi.base import (
     items,
     keys,
 )
-from iommi.refinable import RefinableMembers
+from iommi.declarative.dispatch import dispatch
+from iommi.declarative.namespace import (
+    Namespace,
+    setdefaults_path,
+)
+from iommi.refinable import (
+    Prio,
+    RefinableMembers,
+)
 from iommi.sort_after import sort_after
 from iommi.traversable import (
     Traversable,
@@ -33,7 +33,7 @@ class Members(Traversable):
     """
 
     @dispatch
-    def __init__(self, *, _declared_members, unknown_types_fall_through, cls,  **kwargs):
+    def __init__(self, *, _declared_members, unknown_types_fall_through, cls, **kwargs):
         super(Members, self).__init__(**kwargs)
         self._declared_members = _declared_members
         self._unknown_types_fall_through = unknown_types_fall_through
@@ -62,7 +62,7 @@ def refine_done_members(
 
     Example:
 
-    .. code:: python
+    .. code-block:: python
 
         class ArtistTable(Table):
             instrument = Column()  # <- declared member
@@ -122,14 +122,16 @@ def refine_done_members(
                 member_by_name[key] = item
 
     for k, v in items(Namespace(_unapplied_config)):
-        member_by_name[k] = member_by_name[k].refine(**v)
+        member_by_name[k] = member_by_name[k].refine(Prio.member, **v)
         # noinspection PyProtectedMember
         assert member_by_name[k]._name is not None
 
     if extra_member_defaults:
         for k, v in items(Namespace(extra_member_defaults)):
             if k in member_by_name:
-                member_by_name[k] = member_by_name[k].refine_defaults(**v)
+                v = Namespace(v)
+                v.pop('call_target', None)
+                member_by_name[k] = member_by_name[k].refine(Prio.member_defaults, **v)
             else:
                 member_by_name[k] = Namespace(
                     v,
@@ -203,7 +205,14 @@ class MemberBinder(dict):
         _bindable_names = object.__getattribute__(self, '_bindable_names')
         if name in _bindable_names:
             return self[name]
-        return object.__getattribute__(self, name)
+        try:
+            return object.__getattribute__(self, name)
+        except AttributeError as e:
+            raise AttributeError(
+                f"'{self.__class__.__name__}' object has no member '{name}'.\n"
+                f"Available members:\n    "
+                + '\n    '.join(sorted(_bindable_names)) + '\n'
+            ) from e
 
     def __setattr__(self, name, value):
         self[name] = value
