@@ -9,16 +9,7 @@ from urllib.parse import (
     urlparse,
 )
 
-from tri_declarative import (
-    declarative,
-    dispatch,
-    EMPTY,
-    Namespace,
-    Refinable,
-    setdefaults_path,
-    with_meta,
-)
-from tri_struct import Struct
+from iommi.struct import Struct
 
 from iommi import Fragment
 from iommi._web_compat import (
@@ -31,6 +22,13 @@ from iommi.base import (
     items,
     values,
 )
+from iommi.declarative import declarative
+from iommi.declarative.namespace import (
+    EMPTY,
+    Namespace,
+    setdefaults_path,
+)
+from iommi.declarative.with_meta import with_meta
 from iommi.endpoint import path_join
 from iommi.evaluate import evaluate_strict
 from iommi.fragment import Tag
@@ -41,8 +39,10 @@ from iommi.member import (
 from iommi.part import Part
 from iommi.refinable import (
     EvaluatedRefinable,
+    Refinable,
     RefinableMembers,
 )
+from iommi.shortcut import with_defaults
 
 
 class MenuBase(Part, Tag):
@@ -52,11 +52,13 @@ class MenuBase(Part, Tag):
     attrs: Attrs = Refinable()  # attrs is evaluated, but in a special way so gets no EvaluatedRefinable type
     template: Union[str, Template] = EvaluatedRefinable()
 
-    @dispatch(
+    class Meta:
+        sub_menu = EMPTY
+        attrs__class = EMPTY
+        attrs__style = EMPTY
+
+    @with_defaults(
         sort=True,
-        sub_menu=EMPTY,
-        attrs__class=EMPTY,
-        attrs__style=EMPTY,
     )
     def __init__(self, **kwargs):
         super(MenuBase, self).__init__(**kwargs)
@@ -97,13 +99,15 @@ class MenuItem(MenuBase):
     a = Refinable()
     active_class = Refinable()
 
-    @dispatch(
+    class Meta:
+        a = EMPTY
+
+    @with_defaults(
         display_name=lambda menu_item, **_: capitalize(menu_item.iommi_name()).replace('_', ' '),
         regex=lambda menu_item, **_: '^' + str(menu_item.url) if menu_item.url else None,
-        url=lambda menu_item, **_: '/'
-        + path_join(getattr(menu_item.iommi_parent(), 'url', None), menu_item.iommi_name())
-        + '/',
-        a=EMPTY,
+        url=lambda menu_item, **_: (
+            '/' + path_join(getattr(menu_item.iommi_parent(), 'url', None), menu_item.iommi_name()) + '/'
+        ),
     )
     def __init__(self, **kwargs):
         super(MenuItem, self).__init__(**kwargs)
@@ -123,8 +127,11 @@ class MenuItem(MenuBase):
     def __repr__(self):
         r = f'{self._name} -> {self.url}'
         if self.sub_menu:
-            for i in values(self.sub_menu):
-                r += ''.join([f'\n    {x}' for x in repr(i).split('\n')])
+            if isinstance(self.sub_menu, dict):
+                for i in values(self.sub_menu):
+                    r += ''.join([f'\n    {x}' for x in repr(i).split('\n')])
+            else:
+                r += str(self.sub_menu)
         return r
 
     def __html__(self, *, render=None):
@@ -164,6 +171,14 @@ class MenuException(Exception):
     pass
 
 
+def _path_plus_query(url):
+    x = urlparse(str(url))
+    parsed_url = x.path
+    if x.query:
+        parsed_url += f'?{x.query}'
+    return parsed_url
+
+
 @with_meta
 @declarative(MenuItem, '_sub_menu_dict', add_init_kwargs=False)
 class Menu(MenuBase):
@@ -172,7 +187,7 @@ class Menu(MenuBase):
 
     Example:
 
-    .. code:: python
+    .. code-block:: python
 
         menu = Menu(
             sub_menu=dict(
@@ -189,9 +204,11 @@ class Menu(MenuBase):
 
     items_container = Refinable()
 
-    @dispatch(
+    class Meta:
+        items_container = EMPTY
+
+    @with_defaults(
         sort=False,
-        items_container=EMPTY,
     )
     def __init__(self, **kwargs):
         super(Menu, self).__init__(**kwargs)
@@ -217,7 +234,8 @@ class Menu(MenuBase):
 
     def on_bind(self):
         super(Menu, self).on_bind()
-        self.set_active(current_path=self.get_request().path)
+
+        self.set_active(current_path=_path_plus_query(self.get_request().build_absolute_uri()))
 
     def own_evaluate_parameters(self):
         return dict(menu=self)
@@ -255,7 +273,7 @@ class Menu(MenuBase):
                 if sub_item.url is None or '://' in sub_item.url:
                     continue
 
-                parsed_url = urlparse(str(sub_item.url)).path
+                parsed_url = _path_plus_query(sub_item.url)
 
                 if current_path.startswith(parsed_url):
                     parts = PurePosixPath(unquote(parsed_url)).parts
