@@ -4,15 +4,6 @@ from typing import (
     Union,
 )
 
-from tri_declarative import (
-    declarative,
-    dispatch,
-    EMPTY,
-    Namespace,
-    Refinable,
-    with_meta,
-)
-
 from iommi._web_compat import (
     format_html,
     template_types,
@@ -22,6 +13,13 @@ from iommi.base import (
     items,
     values,
 )
+from iommi.declarative import declarative
+from iommi.declarative.dispatch import dispatch
+from iommi.declarative.namespace import (
+    EMPTY,
+    Namespace,
+)
+from iommi.declarative.with_meta import with_meta
 from iommi.evaluate import evaluate_strict_container
 from iommi.fragment import (
     build_and_bind_h_tag,
@@ -39,6 +37,7 @@ from iommi.part import (
 )
 from iommi.refinable import (
     EvaluatedRefinable,
+    Refinable,
     RefinableMembers,
 )
 from iommi.traversable import Traversable
@@ -55,15 +54,14 @@ class Page(Part):
     """
     A page is used to compose iommi parts into a bigger whole.
 
-    See the `howto <https://docs.iommi.rocks/en/latest/howto.html#parts-pages>`_ for example usages.
+    See the `howto <https://docs.iommi.rocks/en/latest/cookbook_parts_pages.html#parts-pages>`_ for example usages.
     """
 
     title: str = EvaluatedRefinable()
     member_class: Type[Fragment] = Refinable()
     context = Refinable()  # context is evaluated, but in a special way so gets no EvaluatedRefinable type
-    h_tag: Union[
-        Fragment, str
-    ] = Refinable()  # h_tag is evaluated, but in a special way so gets no EvaluatedRefinable type
+    # h_tag is evaluated, but in a special way so gets no EvaluatedRefinable type
+    h_tag: Union[Fragment, str] = Refinable()
     parts: Dict[str, PartType] = RefinableMembers()
 
     class Meta:
