@@ -1,3 +1,4 @@
+import inspect
 import json
 from abc import abstractmethod
 from typing import (
@@ -6,13 +7,6 @@ from typing import (
     Union,
 )
 
-from tri_declarative import (
-    dispatch,
-    EMPTY,
-    Namespace,
-    Refinable,
-)
-
 from iommi._web_compat import (
     get_template_from_string,
     HttpResponse,
@@ -29,6 +23,11 @@ from iommi.debug import (
     get_instantiated_at_info,
     iommi_debug_on,
 )
+from iommi.declarative.dispatch import dispatch
+from iommi.declarative.namespace import (
+    EMPTY,
+    Namespace,
+)
 from iommi.endpoint import (
     DISPATCH_PATH_SEPARATOR,
     Endpoint,
@@ -40,9 +39,9 @@ from iommi.member import (
     bind_members,
     refine_done_members,
 )
+from iommi.shortcut import with_defaults
 from iommi.style import (
-    get_iommi_style_name,
-    get_style,
+    get_style_object,
 )
 from iommi.traversable import (
     Traversable,
@@ -54,6 +53,7 @@ from ._web_compat import (
 )
 from .refinable import (
     EvaluatedRefinable,
+    Refinable,
     RefinableMembers,
 )
 from .sort_after import sort_after
@@ -63,29 +63,28 @@ class Part(Traversable):
     """
     `Part` is the base class for parts of a page that can be rendered as html, and can respond to ajax and post.
 
-    See the `howto <https://docs.iommi.rocks/en/latest/howto.html#parts-pages>`_ for example usages.
+    See the `howto <https://docs.iommi.rocks/en/latest/cookbook_parts_pages.html#parts-pages>`_ for example usages.
     """
 
     include: bool = Refinable()  # This is evaluated, but first and in a special way
     after: Union[int, str] = EvaluatedRefinable()
     extra: Dict[str, Any] = Refinable()
-    extra_evaluated: Dict[
-        str, Any
-    ] = Refinable()  # not EvaluatedRefinable because this is an evaluated container so is special
+    # not EvaluatedRefinable because this is an evaluated container so is special
+    extra_evaluated: Dict[str, Any] = Refinable()
     endpoints: Namespace = RefinableMembers()
     # Only the assets used by this part
     assets: Namespace = RefinableMembers()
 
-    @dispatch(
-        extra=EMPTY,
+    class Meta:
+        extra = EMPTY
+
+    @with_defaults(
         include=True,
     )
-    def __init__(self, **kwargs):
+    def __init__(self, _collect_instantiated_at_info=True, **kwargs):
         super(Part, self).__init__(**kwargs)
 
-        if iommi_debug_on():
-            import inspect
-
+        if _collect_instantiated_at_info:
             frame = inspect.currentframe()
             self._instantiated_at_info = get_instantiated_at_info(frame.f_back)
 
@@ -119,7 +118,7 @@ class Part(Traversable):
         return result
 
     @dispatch
-    def render_to_response(self, **kwargs):
+    def perform_dispatch(self, **kwargs):
         request = self.get_request()
         req_data = request_data(request)
 
@@ -163,6 +162,14 @@ class Part(Traversable):
             if request.method == 'POST':
                 assert False, 'This request was a POST, but there was no dispatch command present.'
 
+        return None
+
+    @dispatch
+    def render_to_response(self, **kwargs):
+        dispatch = self.perform_dispatch(**kwargs)
+        if dispatch is not None:
+            return dispatch
+
         response = HttpResponse(render_root(part=self, **kwargs))
         response.iommi_part = self
         return response
@@ -197,8 +204,7 @@ def get_title(part):
 )
 def render_root(*, part, context, **render):
     assert part._is_bound
-    root_style_name = get_iommi_style_name(part)
-    root_style = get_style(root_style_name)
+    root_style = get_style_object(part)
     template_name = root_style.base_template
     content_block_name = root_style.content_block
 
@@ -208,8 +214,8 @@ def render_root(*, part, context, **render):
 
     assets = part.iommi_collected_assets()
 
-    assert template_name, f"{root_style_name} doesn't have a base_template defined"
-    assert content_block_name, f"{root_style_name} doesn't have a content_block defined"
+    assert template_name, f"{root_style} doesn't have a base_template defined"
+    assert content_block_name, f"{root_style} doesn't have a content_block defined"
 
     title = get_title(part)
 
