@@ -31,30 +31,16 @@ from pyparsing import (
     Word,
     ZeroOrMore,
 )
-from tri_declarative import (
-    class_shortcut,
-    declarative,
-    dispatch,
-    EMPTY,
-    getattr_path,
-    Namespace,
-    Refinable,
-    refinable,
-    setdefaults_path,
-    Shortcut,
-    with_meta,
-)
-from tri_struct import Struct
+from iommi.struct import Struct
 
-from iommi import (
-    Action,
-    Fragment,
-)
 from iommi._web_compat import (
     render_template,
     Template,
     ValidationError,
 )
+from iommi.action import (
+    Action,
+)
 from iommi.base import (
     items,
     keys,
@@ -63,6 +49,15 @@ from iommi.base import (
     NOT_BOUND_MESSAGE,
     values,
 )
+from iommi.declarative import declarative
+from iommi.declarative.dispatch import dispatch
+from iommi.declarative.namespace import (
+    EMPTY,
+    getattr_path,
+    Namespace,
+    setdefaults_path,
+)
+from iommi.declarative.with_meta import with_meta
 from iommi.endpoint import path_join
 from iommi.evaluate import evaluate
 from iommi.form import (
@@ -75,6 +70,9 @@ from iommi.form import (
     int_parse,
     time_parse,
 )
+from iommi.fragment import (
+    Fragment,
+)
 from iommi.from_model import (
     AutoConfig,
     create_members_from_model,
@@ -92,8 +90,15 @@ from iommi.part import (
 )
 from iommi.refinable import (
     EvaluatedRefinable,
+    Prio,
+    Refinable,
+    refinable,
     RefinableMembers,
 )
+from iommi.shortcut import (
+    Shortcut,
+    with_defaults,
+)
 
 
 class QueryException(Exception):
@@ -232,7 +237,7 @@ class Filter(Part):
     is_valid_filter = Refinable()
     query_name = EvaluatedRefinable()
 
-    @dispatch(
+    @with_defaults(
         query_operator_for_field='=',
         attr=MISSING,
         search_fields=MISSING,
@@ -251,6 +256,9 @@ class Filter(Part):
         super(Filter, self).__init__(**kwargs)
 
     def on_refine_done(self):
+        if 'choice' in getattr(self, 'iommi_shortcut_stack', []):
+            assert self.iommi_namespace.get('choices') is not None, 'To use Filter.choice, you must pass the choices list'
+
         model_field = self.model_field
         if model_field and model_field.remote_field:
             self.model = model_field.remote_field.model
@@ -316,70 +324,65 @@ class Filter(Part):
         )
 
     @classmethod
-    @class_shortcut(
+    @with_defaults(
         field__call_target__attribute='text',
         query_operator_for_field=':',
     )
-    def text(cls, call_target=None, **kwargs):
-        return call_target(**kwargs)
+    def text(cls, **kwargs):
+        return cls(**kwargs)
 
     @classmethod
-    @class_shortcut(
-        call_target__attribute='text',
-    )
-    def textarea(cls, call_target=None, **kwargs):
-        return call_target(**kwargs)
+    @with_defaults
+    def textarea(cls, **kwargs):
+        return cls.text(**kwargs)
 
     @classmethod
-    @class_shortcut(
+    @with_defaults(
         query_operator_to_q_operator=case_sensitive_query_operator_to_q_operator,
     )
-    def case_sensitive(cls, call_target=None, **kwargs):
-        return call_target(**kwargs)
+    def case_sensitive(cls, **kwargs):
+        return cls(**kwargs)
 
     @classmethod
-    @class_shortcut(
+    @with_defaults(
         field__call_target__attribute='choice',
     )
-    def choice(cls, call_target=None, **kwargs):
+    def choice(cls, **kwargs):
         """
         Field that has one value out of a set.
         :type choices: list
         """
-        assert 'choices' in kwargs, 'To use Filter.choice, you must pass the choices list'
-        setdefaults_path(
-            kwargs,
-            dict(
-                field__choices=kwargs.get('choices'),
-            ),
+        instance = cls(**kwargs)
+        instance = instance.refine(
+            Prio.shortcut,
+            field__choices=kwargs.get('choices'),
         )
-        return call_target(**kwargs)
+        return instance
 
     @classmethod
-    @class_shortcut(
+    @with_defaults(
         field__call_target__attribute='multi_choice',
     )
-    def multi_choice(cls, call_target=None, **kwargs):
+    def multi_choice(cls, **kwargs):
         """
         Field that has one value out of a set.
         :type choices: list
         """
-        setdefaults_path(
-            kwargs,
-            dict(
-                field__choices=kwargs.get('choices'),
-            ),
+        instance = cls(**kwargs)
+        instance = instance.refine(
+            Prio.shortcut,
+            field__choices=kwargs.get('choices'),
         )
-        return call_target(**kwargs)
+        return instance
 
     @classmethod
-    @class_shortcut(
+    @with_defaults(
         field__call_target__attribute='choice_queryset',
         query_operator_to_q_operator=lambda op: 'exact',
         value_to_q=choice_queryset_value_to_q,
         is_valid_filter=choice_queryset__is_valid_filter,
     )
-    def choice_queryset(cls, choices: QuerySet, call_target=None, **kwargs):
+    def choice_queryset(cls, choices: QuerySet = None, **kwargs):
         """
         Field that has one value out of a set.
         """
@@ -397,128 +400,132 @@ class Filter(Part):
                 choices=choices,
             ),
         )
-        return call_target(**kwargs)
+        return cls(**kwargs)
 
     @classmethod
-    @class_shortcut(
-        call_target__attribute="choice_queryset",
+    @with_defaults(
         field__call_target__attribute='multi_choice_queryset',
     )
-    def multi_choice_queryset(cls, call_target=None, **kwargs):
-        return call_target(**kwargs)
+    def multi_choice_queryset(cls, **kwargs):
+        return cls.choice_queryset(**kwargs)
 
     @classmethod
-    @class_shortcut(
+    @with_defaults(
         field__call_target__attribute='boolean',
         parse=bool_parse,
         unary=True,
         query_operator_to_q_operator=boolean__query_operator_to_q_operator,
     )
-    def boolean(cls, call_target=None, **kwargs):
-        return call_target(**kwargs)
+    def boolean(cls, **kwargs):
+        return cls(**kwargs)
 
     @classmethod
-    @class_shortcut(
+    @with_defaults(
         field__call_target__attribute='boolean_tristate',
         parse=boolean_tristate__parse,
         query_operator_to_q_operator=boolean__query_operator_to_q_operator,
         unary=True,
     )
-    def boolean_tristate(cls, call_target=None, **kwargs):
-        return call_target(**kwargs)
+    def boolean_tristate(cls, **kwargs):
+        return cls(**kwargs)
 
     @classmethod
-    @class_shortcut(
+    @with_defaults(
+        field__call_target__attribute='number',
+    )
+    def number(cls, **kwargs):
+        return cls(**kwargs)
+
+    @classmethod
+    @with_defaults(
         field__call_target__attribute='integer',
         parse=int_parse,
     )
-    def integer(cls, call_target=None, **kwargs):
-        return call_target(**kwargs)
+    def integer(cls, **kwargs):
+        return cls.number(**kwargs)
 
     @classmethod
-    @class_shortcut(
+    @with_defaults(
         field__call_target__attribute='float',
         parse=float_parse,
     )
-    def float(cls, call_target=None, **kwargs):
-        return call_target(**kwargs)
+    def float(cls, **kwargs):
+        return cls.number(**kwargs)
 
     @classmethod
-    @class_shortcut(
+    @with_defaults(
         field__call_target__attribute='url',
     )
-    def url(cls, call_target=None, **kwargs):
-        return call_target(**kwargs)
+    def url(cls, **kwargs):
+        return cls(**kwargs)
 
     @classmethod
-    @class_shortcut(
+    @with_defaults(
         field__call_target__attribute='time',
         parse=time_parse,
     )
-    def time(cls, call_target=None, **kwargs):
-        return call_target(**kwargs)
+    def time(cls, **kwargs):
+        return cls(**kwargs)
 
     @classmethod
-    @class_shortcut(
+    @with_defaults(
         field__call_target__attribute='datetime',
         parse=datetime_parse,
     )
-    def datetime(cls, call_target=None, **kwargs):
-        return call_target(**kwargs)
+    def datetime(cls, **kwargs):
+        return cls(**kwargs)
 
     @classmethod
-    @class_shortcut(
+    @with_defaults(
         field__call_target__attribute='date',
         parse=date_parse,
     )
-    def date(cls, call_target=None, **kwargs):
-        return call_target(**kwargs)
+    def date(cls, **kwargs):
+        return cls(**kwargs)
 
     @classmethod
-    @class_shortcut(
+    @with_defaults(
         field__call_target__attribute='email',
     )
-    def email(cls, call_target=None, **kwargs):
-        return call_target(**kwargs)
+    def email(cls, **kwargs):
+        return cls(**kwargs)
 
     @classmethod
-    @class_shortcut(
+    @with_defaults(
         field__call_target__attribute='decimal',
     )
-    def decimal(cls, call_target=None, **kwargs):
-        return call_target(**kwargs)
+    def decimal(cls, **kwargs):
+        return cls.number(**kwargs)
 
     @classmethod
-    @class_shortcut(
+    @with_defaults(
         field__call_target__attribute='file',
     )
-    def file(cls, call_target=None, **kwargs):
-        return call_target(**kwargs)
+    def file(cls, **kwargs):
+        return cls(**kwargs)
 
     @classmethod
-    @class_shortcut(
-        call_target__attribute='choice_queryset',
+    @with_defaults(
         field__call_target__attribute='foreign_key',
     )
-    def foreign_key(cls, model_field, call_target, **kwargs):
+    def foreign_key(cls, model_field, **kwargs):
         setdefaults_path(
             kwargs,
             choices=model_field.foreign_related_fields[0].model.objects.all(),
         )
-        return call_target(model_field=model_field, **kwargs)
+        return cls.choice_queryset(model_field=model_field, **kwargs)
 
     @classmethod
-    @class_shortcut(
-        call_target__attribute='multi_choice_queryset',
+    @with_defaults(
         field__call_target__attribute='many_to_many',
     )
-    def many_to_many(cls, call_target, model_field, **kwargs):
+    def many_to_many(cls, model_field, **kwargs):
         setdefaults_path(
             kwargs,
             choices=model_field.remote_field.model.objects.all(),
             extra__django_related_field=True,
         )
-        return call_target(model_field=model_field, **kwargs)
+        return cls.multi_choice_queryset(model_field=model_field, **kwargs)
 
 
 Filter.value_to_q.iommi_needs_attr = True
@@ -549,7 +556,7 @@ class QueryAutoConfig(AutoConfig):
 class Advanced(Fragment):
     toggle: Namespace = Refinable()
 
-    @dispatch(
+    @with_defaults(
         toggle__call_target=Action,
         toggle__attrs__href='#',
         toggle__attrs__class__iommi_query_toggle_simple_mode=True,
@@ -576,7 +583,7 @@ class Query(Part):
     """
     Declare a query language. Example:
 
-    .. code:: python
+    .. code-block:: python
 
         class CarQuery(Query):
             make = Filter.choice(choices=['Toyota', 'Volvo', 'Ford'])
@@ -604,11 +611,11 @@ class Query(Part):
     class Meta:
         member_class = Filter
         form_class = Form
+        filters = EMPTY
+        auto = EMPTY
 
-    @dispatch(
+    @with_defaults(
         endpoints__errors__func=default_endpoint__errors,
-        filters=EMPTY,
-        auto=EMPTY,
         form__attrs={'data-iommi-errors': lambda query, **_: query.endpoints.errors.iommi_path},
         form_container__call_target=Fragment,
         form_container__tag='span',
@@ -623,7 +630,7 @@ class Query(Part):
 
         if self.auto:
             auto = QueryAutoConfig(**self.auto).refine_done(parent=self)
-            model, rows, filters = self._from_model(
+            model, rows, filters_from_auto = self._from_model(
                 model=auto.model,
                 rows=auto.rows,
                 include=auto.include,
@@ -640,15 +647,15 @@ class Query(Part):
 
             if self.rows is None:
                 self.rows = rows
-
-            self.filters.update(filters)
+        else:
+            filters_from_auto = None
 
         self.model, self.rows = model_and_rows(self.model, self.rows)
 
         self.query_advanced_value = None
         self.query_error = None
 
-        refine_done_members(self, name='filters', members_from_namespace=self.filters, members_from_declared=self.get_declared('_filters_dict'), cls=self.get_meta().member_class)
+        refine_done_members(self, name='filters', members_from_namespace=self.filters, members_from_declared=self.get_declared('_filters_dict'), members_from_auto=filters_from_auto, cls=self.get_meta().member_class)
 
         self._on_refine_done_form()
 
@@ -677,14 +684,20 @@ class Query(Part):
         field_class = self.get_meta().form_class.get_meta().member_class
 
         declared_fields = Struct()
-        declared_fields[FREETEXT_SEARCH_NAME] = field_class(
-            _name=FREETEXT_SEARCH_NAME,
-            display_name=gettext('Search'),
-            required=False,
-            include=any(filter.freetext for filter in values(self.iommi_namespace.filters)),
-            help__include=False,
-            **self.iommi_namespace.form.get('fields', {}).get(FREETEXT_SEARCH_NAME, {}),
-        )
+
+        freetext_search_config = self.iommi_namespace.form.get('fields', {}).get(FREETEXT_SEARCH_NAME, {})
+        if freetext_search_config is not None:
+            declared_fields[FREETEXT_SEARCH_NAME] = field_class(
+                **setdefaults_path(
+                    Namespace(),
+                    freetext_search_config,
+                    _name=FREETEXT_SEARCH_NAME,
+                    display_name=gettext('Search'),
+                    required=False,
+                    include=lambda query, **_: any(filter.freetext for filter in values(query.filters)),
+                    help__include=False,
+                )
+            )
 
         for name, filter in items(self.iommi_namespace.filters):
             declared_fields[name] = setdefaults_path(
@@ -695,8 +708,7 @@ class Query(Part):
                 model_field=filter.model_field,
                 attr=name if filter.attr is MISSING else filter.attr,
                 help__include=False,
-                # TODO: this isn't right, freetext can be a callable
-                include=filter.include and not filter.freetext,
+                include=lambda query, field, **_: field.iommi_name() in query.filters and not query.filters[field.iommi_name()].freetext,
             )
 
         # Remove fields from the form that correspond to non-included filters
@@ -706,7 +718,7 @@ class Query(Part):
                 continue
             # We need to check if it's in declared_filters first, otherwise we remove any injected fields
             if name in declared_filters and name not in self.iommi_namespace.filters:
-                declared_fields[name] = field.refine(include=False)
+                declared_fields[name] = field.refine_from_query(include=False)
 
         form_args = self.form
 
@@ -1004,6 +1016,7 @@ class Query(Part):
                             ]
                         )
                     )
+                    assert result[-1] != '()'
             return ' and '.join(result)
         else:
             return ''
