@@ -60,13 +60,15 @@ def linkify(s):
     from iommi.debug import src_debug_url_builder
     r = []
     for line in s.split('\n'):
-        match = re.search(r'File "(.*)", line (\d+), in (.*)', line)
+        match = re.search(r'( *)File "(.*)", line (\d+), in (.*)', line)
         if match:
-            filename = match.group(1)
-            lineno = int(match.group(2))
-            function_name = match.group(3)
+            starting_space = match.group(1)
+            filename = match.group(2)
+            lineno = int(match.group(3))
+            function_name = match.group(4)
             r.append(format_html(
-                'File "<a href="{}">{}</a> ", line {}, in {}\n',
+                '{}File "<a href="{}">{}</a> ", line {}, in {}\n',
+                starting_space,
                 src_debug_url_builder(filename, lineno),
                 filename,
                 lineno,
@@ -109,9 +111,9 @@ class Middleware:
                     total_duration = float(sum(x['duration'] for x in iommi_sql_debug_log))
                     result = [
                         '''
-                        <style> 
-                            a, span { 
-                                box-sizing: border-box; 
+                        <style>
+                            a, span {
+                                box-sizing: border-box;
                             }
                             @media (prefers-color-scheme: dark) {
                                 html {
@@ -121,7 +123,7 @@ class Middleware:
                                 b {
                                     color: white;
                                 }
-                            } 
+                            }
                             </style>
                         ''',
                         f'{len(iommi_sql_debug_log)} queries, {total_duration:.3} seconds total<br><br>',
@@ -157,7 +159,6 @@ class Middleware:
                         result.append(format_sql(sql, duration=x['duration']))
                         if sql_trace == SQL_DEBUG_LEVEL_ALL_WITH_STACKS:
                             result.append('\n\n')
-                            # TODO: make links!
                             result.append(linkify(x['stack']))
                         result.append('\n\n')
                     result.append('</pre>')
@@ -261,10 +262,11 @@ def set_sql_debug(new_state, validate=True):
 
 
 def get_sql_debug():
-    result = getattr(state, 'sql_debug', None)
-    if result is not None:
+    sentinel = object()
+    result = getattr(state, 'sql_debug', sentinel)
+    if result is not sentinel:
         return result
-    return getattr(settings, 'SQL_DEBUG', None)
+    return getattr(settings, 'SQL_DEBUG', SQL_DEBUG_LEVEL_WORST)
 
 
 @contextmanager
@@ -315,6 +317,9 @@ def sql_debug_trace_sql(sql, params=None, **kwargs):
 
     kwargs['sql'] = True
     sql_debug(sql, extra=kwargs)
+
+    if get_sql_debug() == SQL_DEBUG_LEVEL_ALL_WITH_STACKS:
+        sql_debug('\n', extra=kwargs)
     return sql
 
 
@@ -329,12 +334,25 @@ def sql_debug_total_time():
 
 def format_clickable_filename(file_name, line, fn, extra=None):
     if not extra:
-        extra = linecache.getline(file_name, line)
+        if line is not None:
+            extra = linecache.getline(file_name, line)
+
+    if extra is None:
+        extra = '<unknown>'
+    if file_name is None:
+        file_name = '<unknown>'
+    if line is None:
+        line = '<unknown>'
+    if fn is None:
+        fn = '<unknown>'
 
     return f'  File "{file_name}", line {line}, in {fn} => {extra.strip()}'
 
 
 def sql_debug_format_stack_trace(frame):
+    if frame is None:
+        return None
+
     base_path = os.path.abspath(os.path.join(settings.BASE_DIR, '..')) + "/"
     msg = []
     skip_template_code = False
@@ -428,9 +446,12 @@ def sql_debug_last_call(response):
 
 class CursorDebugWrapper(django_db_utils.CursorWrapper):
     def _execute_and_log(self, *, f, **kwargs):
-        frame = sys._getframe().f_back.f_back
-        while "django/db" in frame.f_code.co_filename:
-            frame = frame.f_back
+        if get_sql_debug() is not None:
+            frame = sys._getframe().f_back.f_back
+            while "django/db" in frame.f_code.co_filename:
+                frame = frame.f_back
+        else:
+            frame = None
 
         start = monotonic()
         try:
@@ -467,4 +488,3 @@ def make_debug_cursor(self, cursor):
 
 
 connections[DEFAULT_DB_ALIAS].__class__.make_debug_cursor = make_debug_cursor
-
