@@ -4,74 +4,28 @@ from typing import (
     Any,
     List,
     Type,
+    Union,
 )
 
-from tri_declarative import (
-    dispatch,
-    EMPTY,
-    get_shortcuts_by_name,
-    Namespace,
-    RefinableObject,
-    setdefaults_path,
-)
+from django.conf import settings
 
-from iommi._web_compat import settings
 from iommi.base import (
     items,
     keys,
 )
+from iommi.declarative.dispatch import dispatch
+from iommi.declarative.namespace import (
+    EMPTY,
+    Namespace,
+)
 from iommi.refinable import RefinableObject
+from iommi.shortcut import get_shortcuts_by_name
 
 DEFAULT_STYLE = 'bootstrap'
 
 
-def get_iommi_style_name(obj: Any) -> str:
-    while obj is not None:
-        if obj.iommi_namespace.get('iommi_style') is not None:
-            return obj.iommi_namespace.iommi_style
-        obj = obj.iommi_parent()
-    return getattr(settings, 'IOMMI_DEFAULT_STYLE', DEFAULT_STYLE)
-
-
 def get_style_object(obj: Any) -> 'Style':
-    # Step 1: build the stack
-    stack = []
-    o = obj
-    while o is not None:
-        if o.iommi_style is not None:
-            stack.append(o.iommi_style)
-            if isinstance(o.iommi_style, Style):
-                break
-        o = o.iommi_parent()
-
-    # Step 2: resolve names in the stack
-    stack.append(get_style(getattr(settings, 'IOMMI_DEFAULT_STYLE', DEFAULT_STYLE)))
-    stack.reverse()
-
-    if len(stack) == 1:
-        return stack[0]
-
-    def resolve(x, previous_style):
-        if isinstance(x, Style):
-            return x
-
-        assert isinstance(x, str)
-
-        if previous_style is None:
-            return get_style(x)
-
-        result = previous_style.resolve(x)
-        if result is None:
-            return get_style(x)
-
-        assert isinstance(result, Style)
-        return result
-
-    resolved_stack = []
-    for i, x in enumerate(stack):
-        resolved_stack.append(resolve(x, resolved_stack[i - 1] if i else None))
-
-    return resolved_stack[-1]
+    return get_global_style(obj.iommi_style)
 
 
 def _style_name_for_class(cls):
@@ -101,7 +55,7 @@ class Style:
         sub_styles=EMPTY,
     )
     def __init__(
-        self, *bases, base_template=None, content_block=None, assets=None, root=None, internal=False, sub_styles=None, **kwargs
+        self, *bases, base_template=None, content_block=None, root=None, internal=False, sub_styles=None, **kwargs
     ):
         self.name = None
         self.internal = internal
@@ -121,75 +75,73 @@ class Style:
                     self.content_block = base.content_block
                     break
 
-        if assets:
-            from iommi.debug import iommi_debug_on
-
-            if iommi_debug_on():
-                print(
-                    "Warning: The preferred way to add top level assets config to a Style is via the root argument. "
-                    "I.e. assets__* becomes root__assets__*"
-                )
-            setdefaults_path(root, assets=assets)
-
         self.root = {k: v for k, v in items(Namespace(*(base.root for base in bases), root)) if v is not None}
         self.config = Namespace(*[x.config for x in bases], recursive_namespace(kwargs))
-        self.sub_styles = {
-            k: Style(self, **v)
-            for k, v in items(sub_styles)
-        }
 
-    def component(self, obj, is_root=False):
+        sub_style_names = list(sub_styles)
+        for base in bases:
+            for n in base.sub_styles:
+                if n not in sub_style_names:
+                    sub_style_names.append(n)
+
+        self.sub_styles = {}
+        for k in sub_style_names:
+            v = sub_styles.get(k, {})
+            if isinstance(v, Style):
+                self.sub_styles[k] = v
+            else:
+                sub_style_bases = [self]
+                for base in bases:
+                    sub_style_base = base.sub_styles.get(k)
+                    if sub_style_base:
+                        sub_style_bases.append(sub_style_base)
+                self.sub_styles[k] = Style(*sub_style_bases, **v)
+
+        for name, sub_style in items(self.sub_styles):
+            sub_style.name = name
+
+    def resolve(self, obj, is_root=False):
         """
-        Calculate the namespace of additional argument that should be applied
+        Calculate the namespaces of additional argument that should be applied
         to the given object. If is_root is set to True, assets might also be
         added to the namespace.
         """
-        result = Namespace()
+        result = []
 
-        # TODO: is this wrong? Should it take classes first, then loop through shortcuts?
         for class_name in class_names_for(type(obj)):
             if class_name in self.config:
-                config = Namespace(self.config.get(class_name, {}))
-                shortcuts_config = Namespace(config.pop('shortcuts', {}))
-                result.update(config)
+                config = dict(self.config.get(class_name))
+                shortcuts_config = config.pop('shortcuts', {})
+                if config:
+                    result.append(config)
 
-                for shortcut_name in reversed(getattr(obj, '__tri_declarative_shortcut_stack', [])):
-                    result = Namespace(result, shortcuts_config.get(shortcut_name, {}))
+                for shortcut_name in reversed(getattr(obj, 'iommi_shortcut_stack', [])):
+                    config = shortcuts_config.get(shortcut_name)
+                    if config:
+                        result.append(config)
 
-        if is_root:
-            result = Namespace(result, self.root)
+        if is_root and self.root:
+            result.append(self.root)
 
         return result
 
     def __repr__(self):
         return f'<Style: {self.name}>'
 
-    def resolve(self, sub_style_name):
-        result = self.sub_styles.get(sub_style_name)
-        if result:
-            return result
-
-        for base in self.bases:
-            result = base.resolve(sub_style_name)
-            if result:
-                return result
-
-        return None
-
 
 _styles = {}
 
 
-def register_style(name, conf):
+def register_style(name, style):
     assert name not in _styles, f'{name} is already registered'
-    assert conf.name is None
-    conf.name = name
-    _styles[name] = conf
+    assert style.name is None
+    style.name = name
+    _styles[name] = style
 
     @contextmanager
     def _unregister():
         try:
-            yield
+            yield style
         finally:
             unregister_style(name)
     return _unregister()
@@ -200,25 +152,25 @@ def unregister_style(name):
     del _styles[name]
 
 
-def get_style(name):
+def get_global_style(name):
     if isinstance(name, Style):
         return name
     try:
         return _styles[name]
     except KeyError:
         style_names = "\n    ".join(_styles.keys())
+
+        if not style_names:
+            'No styles registered! Did you forget to add iommi to INSTALLED_APPS?'
+
         raise Exception(
-            f'''No registered style {name}. Register a style with register_style().
+            f'''No registered iommi style {name}. Register a style with register_style().
 
 Available styles:
     {style_names}'''
         ) from None
 
 
-def get_style_data_for_object(style_object, obj, is_root):
-    return style_object.component(obj, is_root)
-
-
 class InvalidStyleConfigurationException(Exception):
     pass
 
@@ -330,3 +282,21 @@ def validate_styles(*, additional_classes: List[Type] = None, default_classes=No
         if invalid_shortcut_names_str:
             invalid_shortcut_names_str = 'Invalid shortcut names:\n' + invalid_shortcut_names_str
         raise InvalidStyleConfigurationException('\n\n'.join([invalid_class_names_str, invalid_shortcut_names_str]))
+
+
+def resolve_style(iommi_style: Union[str, Style], enclosing_style: Style = None):
+    if isinstance(iommi_style, Style):
+        return iommi_style
+
+    if not enclosing_style:
+        default_style = get_global_style(getattr(settings, 'IOMMI_DEFAULT_STYLE', DEFAULT_STYLE))
+        enclosing_style = default_style
+
+    if iommi_style is None:
+        return enclosing_style
+
+    sub_style = enclosing_style.sub_styles.get(iommi_style)
+    if sub_style is not None:
+        return sub_style
+
+    return get_global_style(iommi_style)
