@@ -5,40 +5,34 @@ from typing import (
     List,
 )
 
-from django.conf import settings
-from tri_declarative import (
-    Namespace,
-    Refinable,
-)
-from tri_struct import Struct
+from iommi.struct import Struct
 
 from iommi.attrs import evaluate_attrs
 from iommi.base import (
     items,
     NOT_BOUND_MESSAGE,
 )
+from iommi.declarative.namespace import Namespace
 from iommi.evaluate import (
     evaluate_members,
     evaluate_strict,
     evaluate_strict_container,
 )
+from iommi.path import decode_path_components
 from iommi.refinable import (
     evaluated_refinable,
     EvaluatedRefinable,
     is_evaluated_refinable,
+    Prio,
+    Refinable,
     RefinableMembers,
     RefinableObject,
 )
-from iommi.style import (
-    DEFAULT_STYLE,
-    get_style,
-    get_style_data_for_object,
-    Style,
-)
+from iommi.style import Style
 
 # Backward compatible definition
-EvaluatedRefinable = EvaluatedRefinable
-evaluated_refinable = evaluated_refinable
+EvaluatedRefinable = EvaluatedRefinable  # pragma: no mutate this is just marking the symbols as used
+evaluated_refinable = evaluated_refinable  # pragma: no mutate this is just marking the symbols as used
 
 
 class PathNotFoundException(Exception):
@@ -104,6 +98,9 @@ class Traversable(RefinableObject):
 
     @property
     def iommi_path(self) -> str:
+        if hasattr(self, '_iommi_path_override'):
+            return self._iommi_path_override
+
         long_path = build_long_path(self)
         path_by_long_path = get_path_by_long_path(self)
         path = path_by_long_path.get(long_path)
@@ -119,34 +116,21 @@ class Traversable(RefinableObject):
         assert self._is_bound, NOT_BOUND_MESSAGE
         return build_long_path(self).replace('/', '__')
 
-    def apply_styles(self, parent_style: Style, is_root=True):
-        iommi_style = self.iommi_namespace.get('iommi_style')
+    def apply_style(self, iommi_style: Style, is_root=True):
+        assert iommi_style.__class__.__name__ == "Style", iommi_style.__class__.__name__
 
-        if iommi_style is None:
-            iommi_style = parent_style
+        refinements = iommi_style.resolve(obj=self, is_root=is_root)
 
-        if isinstance(iommi_style, str):
-            if isinstance(parent_style, Style):
-                iommi_style = parent_style.resolve(iommi_style)
-            else:
-                iommi_style = get_style(iommi_style)
-
-        if iommi_style is None:
-            iommi_style = get_style(getattr(settings, 'IOMMI_DEFAULT_STYLE', DEFAULT_STYLE))
-
-        assert iommi_style.__class__.__name__ == "Style"
-
-        style_data = get_style_data_for_object(iommi_style, obj=self, is_root=is_root)
-
-        result = self.refine_defaults(**style_data)
+        result = self
         del self
 
+        if refinements:
+            for refinement in refinements:
+                result = result.refine(Prio.style, **Namespace(refinement))
+        else:
+            result = result.refine(Prio.style)
+
         result.iommi_style = iommi_style
-
-        for k, v in items(result.get_declared('refinable')):
-            if isinstance(v, Traversable):
-                setattr(result, k, v.apply_styles(iommi_style, is_root=False))
-
         return result
 
     def bind(self, *, parent=None, request=None):
@@ -181,6 +165,8 @@ class Traversable(RefinableObject):
         }
         if parent is None:
             evaluate_parameters['request'] = request
+            if hasattr(request, 'iommi_view_params'):
+                evaluate_parameters['params'] = Struct(decode_path_components(request, **request.iommi_view_params))
         result._evaluate_parameters = evaluate_parameters
 
         if hasattr(result, 'include'):
